@@ -73,6 +73,8 @@ all other classes are runtime bindings::
     inject.configure(my_config)
 
 """
+import contextlib
+
 from inject._version import __version__
 
 import inspect
@@ -122,8 +124,9 @@ class ConstructorTypeError(TypeError):
 class Binder(object):
     _bindings: Dict[Binding, Constructor]
 
-    def __init__(self) -> None:
+    def __init__(self, allow_override: bool = False) -> None:
         self._bindings = {}
+        self.allow_override = allow_override
 
     def install(self, config: BinderCallable) -> 'Binder':
         """Install another callable configuration."""
@@ -155,7 +158,10 @@ class Binder(object):
         return self
 
     def bind_to_provider(self, cls: Binding, provider: Provider) -> 'Binder':
-        """Bind a class to a callable instance provider executed for each injection."""
+        """
+        Bind a class to a callable instance provider executed for each injection.
+        A provider can be a normal function or a context manager. Both sync and async are supported.
+        """
         self._check_class(cls)
         if provider is None:
             raise InjectorException('Provider cannot be None, key=%s' % cls)
@@ -171,7 +177,7 @@ class Binder(object):
         if cls is None:
             raise InjectorException('Binding key cannot be None')
 
-        if cls in self._bindings:
+        if not self.allow_override and cls in self._bindings:
             raise InjectorException('Duplicate binding, key=%s' % cls)
 
         if self._is_forward_str(cls):
@@ -197,10 +203,12 @@ class Binder(object):
 class Injector(object):
     _bindings: Dict[Binding, Constructor]
 
-    def __init__(self, config: Optional[BinderCallable] = None, bind_in_runtime: bool = True):
+    def __init__(
+        self, config: Optional[BinderCallable] = None, bind_in_runtime: bool = True, allow_override: bool = False
+    ):
         self._bind_in_runtime = bind_in_runtime
         if config:
-            binder = Binder()
+            binder = Binder(allow_override)
             config(binder)
             self._bindings = binder._bindings
         else:
@@ -320,6 +328,35 @@ class _ParametersInjection(Generic[T]):
     def __init__(self, **kwargs: Any) -> None:
         self._params = kwargs
 
+    @staticmethod
+    def _aggregate_sync_stack(
+            sync_stack: contextlib.ExitStack,
+            provided_params: frozenset[str],
+            kwargs: dict[str, Any]
+    ) -> None:
+        """Extracts context managers, aggregate them in an ExitStack and swap out the param value with results of
+        running __enter__(). The result is equivalent to using `with` multiple times """
+        executed_kwargs = {
+            param: sync_stack.enter_context(inst)
+            for param, inst in kwargs.items()
+            if param not in provided_params and isinstance(inst, contextlib._GeneratorContextManager)
+        }
+        kwargs.update(executed_kwargs)
+
+    @staticmethod
+    async def _aggregate_async_stack(
+            async_stack: contextlib.AsyncExitStack,
+            provided_params: frozenset[str],
+            kwargs: dict[str, Any]
+    ) -> None:
+        """Similar to _aggregate_sync_stack, but for async context managers"""
+        executed_kwargs = {
+            param: await async_stack.enter_async_context(inst)
+            for param, inst in kwargs.items()
+            if param not in provided_params and isinstance(inst, contextlib._AsyncGeneratorContextManager)
+        }
+        kwargs.update(executed_kwargs)
+
     def __call__(self, func: Callable[..., Union[Awaitable[T], T]]) -> Callable[..., Union[Awaitable[T], T]]:
         if sys.version_info.major == 2:
             arg_names = inspect.getargspec(func).args
@@ -337,7 +374,11 @@ class _ParametersInjection(Generic[T]):
                         kwargs[param] = instance(cls)
                 async_func = cast(Callable[..., Awaitable[T]], func)
                 try:
-                    return await async_func(*args, **kwargs)
+                    with contextlib.ExitStack() as sync_stack:
+                        async with contextlib.AsyncExitStack() as async_stack:
+                            self._aggregate_sync_stack(sync_stack, provided_params, kwargs)
+                            await self._aggregate_async_stack(async_stack, provided_params, kwargs)
+                            return await async_func(*args, **kwargs)
                 except TypeError as previous_error:
                     raise ConstructorTypeError(func, previous_error)
 
@@ -352,39 +393,69 @@ class _ParametersInjection(Generic[T]):
                     kwargs[param] = instance(cls)
             sync_func = cast(Callable[..., T], func)
             try:
-                return sync_func(*args, **kwargs)
+                with contextlib.ExitStack() as sync_stack:
+                    self._aggregate_sync_stack(sync_stack, provided_params, kwargs)
+                    return sync_func(*args, **kwargs)
             except TypeError as previous_error:
                 raise ConstructorTypeError(func, previous_error)
         return injection_wrapper
 
 
-def configure(config: Optional[BinderCallable] = None, bind_in_runtime: bool = True) -> Injector:
+def configure(
+    config: Optional[BinderCallable] = None, 
+    bind_in_runtime: bool = True, 
+    allow_override: bool = False,
+    clear: bool = False,
+    once: bool = False
+) -> Injector:
     """Create an injector with a callable config or raise an exception when already configured."""
     global _INJECTOR
 
+    if clear and once:
+        raise InjectorException('clear and once are mutually exclusive, only one can be True')
+
     with _INJECTOR_LOCK:
         if _INJECTOR:
-            raise InjectorException('Injector is already configured')
+            if clear:
+                _clear_injector()
+            elif once:
+                return _INJECTOR
+            else:
+                raise InjectorException('Injector is already configured')
 
-        _INJECTOR = Injector(config, bind_in_runtime=bind_in_runtime)
+        _INJECTOR = Injector(config, bind_in_runtime=bind_in_runtime, allow_override=allow_override)
         logger.debug('Created and configured an injector, config=%s', config)
         return _INJECTOR
 
 
-def configure_once(config: Optional[BinderCallable] = None, bind_in_runtime: bool = True) -> Injector:
-    """Create an injector with a callable config if not present, otherwise, do nothing."""
+def configure_once(
+    config: Optional[BinderCallable] = None, 
+    bind_in_runtime: bool = True, 
+    allow_override: bool = False
+) -> Injector:
+    """Create an injector with a callable config if not present, otherwise, do nothing.
+    
+    Deprecated, use `configure(once=True)` instead.
+    """
     with _INJECTOR_LOCK:
         if _INJECTOR:
             return _INJECTOR
 
-        return configure(config, bind_in_runtime=bind_in_runtime)
+        return configure(config, bind_in_runtime=bind_in_runtime, allow_override=allow_override)
 
 
-def clear_and_configure(config: Optional[BinderCallable] = None, bind_in_runtime: bool = True) -> Injector:
-    """Clear an existing injector and create another one with a callable config."""
+def clear_and_configure(
+    config: Optional[BinderCallable] = None, 
+    bind_in_runtime: bool = True, 
+    allow_override: bool = False
+) -> Injector:
+    """Clear an existing injector and create another one with a callable config.
+    
+    Deprecated, use configure(clear=True) instead.
+    """
     with _INJECTOR_LOCK:
-        clear()
-        return configure(config, bind_in_runtime=bind_in_runtime)
+        _clear_injector()
+        return configure(config, bind_in_runtime=bind_in_runtime, allow_override=allow_override)
 
 
 def is_configured() -> bool:
@@ -394,6 +465,11 @@ def is_configured() -> bool:
 
 
 def clear() -> None:
+    """Clear an existing injector if present."""
+    _clear_injector()
+
+
+def _clear_injector() -> None:
     """Clear an existing injector if present."""
     global _INJECTOR
 
@@ -404,6 +480,7 @@ def clear() -> None:
         _INJECTOR = None
         logger.debug('Cleared an injector')
 
+
 @overload
 def instance(cls: Type[T]) -> T: ...
 
